@@ -4,28 +4,22 @@ The :mod:`sklearn.pls` module implements Partial Least Squares (PLS).
 
 # Author: Edouard Duchesnay <edouard.duchesnay@cea.fr>
 # License: BSD 3 clause
-from distutils.version import LooseVersion
-from sklearn.utils.extmath import svd_flip
-
-from ..base import BaseEstimator, RegressorMixin, TransformerMixin
-from ..utils import check_array, check_consistent_length
-from ..externals import six
 
 import warnings
 from abc import ABCMeta, abstractmethod
+
 import numpy as np
-from scipy import linalg
-from ..utils import arpack
+from scipy.linalg import pinv2, svd
+from scipy.sparse.linalg import svds
+
+from ..base import BaseEstimator, RegressorMixin, TransformerMixin
+from ..utils import check_array, check_consistent_length
+from ..utils.extmath import svd_flip
 from ..utils.validation import check_is_fitted, FLOAT_DTYPES
+from ..externals import six
 
 __all__ = ['PLSCanonical', 'PLSRegression', 'PLSSVD']
 
-import scipy
-pinv2_args = {}
-if LooseVersion(scipy.__version__) >= LooseVersion('0.12'):
-    # check_finite=False is an optimization available only in scipy >=0.12
-    pinv2_args = {'check_finite': False}
-
 
 def _nipals_twoblocks_inner_loop(X, Y, mode="A", max_iter=500, tol=1e-06,
                                  norm_y_weights=False):
@@ -48,11 +42,16 @@ def _nipals_twoblocks_inner_loop(X, Y, mode="A", max_iter=500, tol=1e-06,
             if X_pinv is None:
                 # We use slower pinv2 (same as np.linalg.pinv) for stability
                 # reasons
-                X_pinv = linalg.pinv2(X, **pinv2_args)
+                X_pinv = pinv2(X, check_finite=False)
             x_weights = np.dot(X_pinv, y_score)
         else:  # mode A
             # Mode A regress each X column on y_score
             x_weights = np.dot(X.T, y_score) / np.dot(y_score.T, y_score)
+        # If y_score only has zeros x_weights will only have zeros. In
+        # this case add an epsilon to converge to a more acceptable
+        # solution
+        if np.dot(x_weights.T, x_weights) < eps:
+            x_weights += eps
         # 1.2 Normalize u
         x_weights /= np.sqrt(np.dot(x_weights.T, x_weights)) + eps
         # 1.3 Update x_score: the X latent scores
@@ -60,7 +59,7 @@ def _nipals_twoblocks_inner_loop(X, Y, mode="A", max_iter=500, tol=1e-06,
         # 2.1 Update y_weights
         if mode == "B":
             if Y_pinv is None:
-                Y_pinv = linalg.pinv2(Y, **pinv2_args)  # compute once pinv(Y)
+                Y_pinv = pinv2(Y, check_finite=False)  # compute once pinv(Y)
             y_weights = np.dot(Y_pinv, x_score)
         else:
             # Mode A regress each Y column on x_score
@@ -84,7 +83,7 @@ def _nipals_twoblocks_inner_loop(X, Y, mode="A", max_iter=500, tol=1e-06,
 
 def _svd_cross_product(X, Y):
     C = np.dot(X.T, Y)
-    U, s, Vh = linalg.svd(C, full_matrices=False)
+    U, s, Vh = svd(C, full_matrices=False)
     u = U[:, [0]]
     v = Vh.T[:, [0]]
     return u, v
@@ -148,7 +147,7 @@ class _PLS(six.with_metaclass(ABCMeta), BaseEstimator, TransformerMixin,
 
     mode : "A" classical PLS and "B" CCA. See notes.
 
-    norm_y_weights: boolean, normalize Y weights to one? (default False)
+    norm_y_weights : boolean, normalize Y weights to one? (default False)
 
     algorithm : string, "nipals" or "svd"
         The algorithm used to estimate the weights. It will be called
@@ -190,7 +189,7 @@ class _PLS(six.with_metaclass(ABCMeta), BaseEstimator, TransformerMixin,
     y_rotations_ : array, [q, n_components]
         Y block to latents rotations.
 
-    coef_: array, [p, q]
+    coef_ : array, [p, q]
         The coefficients of the linear model: ``Y = X coef_ + Err``
 
     n_iter_ : array-like
@@ -236,11 +235,11 @@ class _PLS(six.with_metaclass(ABCMeta), BaseEstimator, TransformerMixin,
         Parameters
         ----------
         X : array-like, shape = [n_samples, n_features]
-            Training vectors, where n_samples in the number of samples and
+            Training vectors, where n_samples is the number of samples and
             n_features is the number of predictors.
 
-        Y : array-like of response, shape = [n_samples, n_targets]
-            Target vectors, where n_samples in the number of samples and
+        Y : array-like, shape = [n_samples, n_targets]
+            Target vectors, where n_samples is the number of samples and
             n_targets is the number of response variables.
         """
 
@@ -348,13 +347,13 @@ class _PLS(six.with_metaclass(ABCMeta), BaseEstimator, TransformerMixin,
         # U = Y C(Q'C)^-1 = YC* (W* : q x k matrix)
         self.x_rotations_ = np.dot(
             self.x_weights_,
-            linalg.pinv2(np.dot(self.x_loadings_.T, self.x_weights_),
-                         **pinv2_args))
+            pinv2(np.dot(self.x_loadings_.T, self.x_weights_),
+                  check_finite=False))
         if Y.shape[1] > 1:
             self.y_rotations_ = np.dot(
                 self.y_weights_,
-                linalg.pinv2(np.dot(self.y_loadings_.T, self.y_weights_),
-                             **pinv2_args))
+                pinv2(np.dot(self.y_loadings_.T, self.y_weights_),
+                      check_finite=False))
         else:
             self.y_rotations_ = np.ones(1)
 
@@ -367,8 +366,7 @@ class _PLS(six.with_metaclass(ABCMeta), BaseEstimator, TransformerMixin,
             # Y = X W(P'W)^-1Q' + Err = XB + Err
             # => B = W*Q' (p x q)
             self.coef_ = np.dot(self.x_rotations_, self.y_loadings_.T)
-            self.coef_ = (1. / self.x_std_.reshape((p, 1)) * self.coef_ *
-                          self.y_std_)
+            self.coef_ = self.coef_ * self.y_std_
         return self
 
     def transform(self, X, Y=None, copy=True):
@@ -376,13 +374,13 @@ class _PLS(six.with_metaclass(ABCMeta), BaseEstimator, TransformerMixin,
 
         Parameters
         ----------
-        X : array-like of predictors, shape = [n_samples, p]
-            Training vectors, where n_samples in the number of samples and
-            p is the number of predictors.
+        X : array-like, shape = [n_samples, n_features]
+            Training vectors, where n_samples is the number of samples and
+            n_features is the number of predictors.
 
-        Y : array-like of response, shape = [n_samples, q], optional
-            Training vectors, where n_samples in the number of samples and
-            q is the number of response variables.
+        Y : array-like, shape = [n_samples, n_targets]
+            Target vectors, where n_samples is the number of samples and
+            n_targets is the number of response variables.
 
         copy : boolean, default True
             Whether to copy X and Y, or perform in-place normalization.
@@ -414,9 +412,9 @@ class _PLS(six.with_metaclass(ABCMeta), BaseEstimator, TransformerMixin,
 
         Parameters
         ----------
-        X : array-like of predictors, shape = [n_samples, p]
-            Training vectors, where n_samples in the number of samples and
-            p is the number of predictors.
+        X : array-like, shape = [n_samples, n_features]
+            Training vectors, where n_samples is the number of samples and
+            n_features is the number of predictors.
 
         copy : boolean, default True
             Whether to copy X and Y, or perform in-place normalization.
@@ -434,27 +432,24 @@ class _PLS(six.with_metaclass(ABCMeta), BaseEstimator, TransformerMixin,
         Ypred = np.dot(X, self.coef_)
         return Ypred + self.y_mean_
 
-    def fit_transform(self, X, y=None, **fit_params):
+    def fit_transform(self, X, y=None):
         """Learn and apply the dimension reduction on the train data.
 
         Parameters
         ----------
-        X : array-like of predictors, shape = [n_samples, p]
-            Training vectors, where n_samples in the number of samples and
-            p is the number of predictors.
+        X : array-like, shape = [n_samples, n_features]
+            Training vectors, where n_samples is the number of samples and
+            n_features is the number of predictors.
 
-        Y : array-like of response, shape = [n_samples, q], optional
-            Training vectors, where n_samples in the number of samples and
-            q is the number of response variables.
-
-        copy : boolean, default True
-            Whether to copy X and Y, or perform in-place normalization.
+        y : array-like, shape = [n_samples, n_targets]
+            Target vectors, where n_samples is the number of samples and
+            n_targets is the number of response variables.
 
         Returns
         -------
         x_scores if Y is not given, (x_scores, y_scores) otherwise.
         """
-        return self.fit(X, y, **fit_params).transform(X, y)
+        return self.fit(X, y).transform(X, y)
 
 
 class PLSRegression(_PLS):
@@ -512,7 +507,7 @@ class PLSRegression(_PLS):
     y_rotations_ : array, [q, n_components]
         Y block to latents rotations.
 
-    coef_: array, [p, q]
+    coef_ : array, [p, q]
         The coefficients of the linear model: ``Y = X coef_ + Err``
 
     n_iter_ : array-like
@@ -540,7 +535,9 @@ class PLSRegression(_PLS):
 
     where Xk and Yk are residual matrices at iteration k.
 
-    `Slides explaining PLS <http://www.eigenvector.com/Docs/Wise_pls_properties.pdf>`
+    `Slides explaining
+    PLS <http://www.eigenvector.com/Docs/Wise_pls_properties.pdf>`_
+
 
     For each component k, find weights u, v that optimizes:
     ``max corr(Xk u, Yk v) * std(Xk u) std(Yk u)``, such that ``|u| = 1``
@@ -588,10 +585,11 @@ class PLSRegression(_PLS):
 
     def __init__(self, n_components=2, scale=True,
                  max_iter=500, tol=1e-06, copy=True):
-        _PLS.__init__(self, n_components=n_components, scale=scale,
-                      deflation_mode="regression", mode="A",
-                      norm_y_weights=False, max_iter=max_iter, tol=tol,
-                      copy=copy)
+        super(PLSRegression, self).__init__(
+            n_components=n_components, scale=scale,
+            deflation_mode="regression", mode="A",
+            norm_y_weights=False, max_iter=max_iter, tol=tol,
+            copy=copy)
 
 
 class PLSCanonical(_PLS):
@@ -606,7 +604,11 @@ class PLSCanonical(_PLS):
 
     Parameters
     ----------
-    scale : boolean, scale data? (default True)
+    n_components : int, (default 2).
+        Number of components to keep
+
+    scale : boolean, (default True)
+        Option to scale data
 
     algorithm : string, "nipals" or "svd"
         The algorithm used to estimate the weights. It will be called
@@ -623,8 +625,6 @@ class PLSCanonical(_PLS):
         Whether the deflation should be done on a copy. Let the default
         value to True unless you don't care about side effect
 
-    n_components : int, number of components to keep. (default 2).
-
     Attributes
     ----------
     x_weights_ : array, shape = [p, n_components]
@@ -676,7 +676,8 @@ class PLSCanonical(_PLS):
 
     where Xk and Yk are residual matrices at iteration k.
 
-    `Slides explaining PLS <http://www.eigenvector.com/Docs/Wise_pls_properties.pdf>`
+    `Slides explaining PLS
+    <http://www.eigenvector.com/Docs/Wise_pls_properties.pdf>`_
 
     For each component k, find weights u, v that optimize::
 
@@ -730,10 +731,11 @@ class PLSCanonical(_PLS):
 
     def __init__(self, n_components=2, scale=True, algorithm="nipals",
                  max_iter=500, tol=1e-06, copy=True):
-        _PLS.__init__(self, n_components=n_components, scale=scale,
-                      deflation_mode="canonical", mode="A",
-                      norm_y_weights=True, algorithm=algorithm,
-                      max_iter=max_iter, tol=tol, copy=copy)
+        super(PLSCanonical, self).__init__(
+            n_components=n_components, scale=scale,
+            deflation_mode="canonical", mode="A",
+            norm_y_weights=True, algorithm=algorithm,
+            max_iter=max_iter, tol=tol, copy=copy)
 
 
 class PLSSVD(BaseEstimator, TransformerMixin):
@@ -781,6 +783,18 @@ class PLSSVD(BaseEstimator, TransformerMixin):
         self.copy = copy
 
     def fit(self, X, Y):
+        """Fit model to data.
+
+        Parameters
+        ----------
+        X : array-like, shape = [n_samples, n_features]
+            Training vectors, where n_samples is the number of samples and
+            n_features is the number of predictors.
+
+        Y : array-like, shape = [n_samples, n_targets]
+            Target vectors, where n_samples is the number of samples and
+            n_targets is the number of response variables.
+        """
         # copy since this will contains the centered data
         check_consistent_length(X, Y)
         X = check_array(X, dtype=np.float64, copy=self.copy)
@@ -804,9 +818,9 @@ class PLSSVD(BaseEstimator, TransformerMixin):
         # all the components (C.shape[1]), we have to use another one. Else,
         # let's use arpacks to compute only the interesting components.
         if self.n_components >= np.min(C.shape):
-            U, s, V = linalg.svd(C, full_matrices=False)
+            U, s, V = svd(C, full_matrices=False)
         else:
-            U, s, V = arpack.svds(C, k=self.n_components)
+            U, s, V = svds(C, k=self.n_components)
         # Deterministic output
         U, V = svd_flip(U, V)
         V = V.T
@@ -817,7 +831,19 @@ class PLSSVD(BaseEstimator, TransformerMixin):
         return self
 
     def transform(self, X, Y=None):
-        """Apply the dimension reduction learned on the train data."""
+        """
+        Apply the dimension reduction learned on the train data.
+
+        Parameters
+        ----------
+        X : array-like, shape = [n_samples, n_features]
+            Training vectors, where n_samples is the number of samples and
+            n_features is the number of predictors.
+
+        Y : array-like, shape = [n_samples, n_targets]
+            Target vectors, where n_samples is the number of samples and
+            n_targets is the number of response variables.
+        """
         check_is_fitted(self, 'x_mean_')
         X = check_array(X, dtype=np.float64)
         Xr = (X - self.x_mean_) / self.x_std_
@@ -830,21 +856,21 @@ class PLSSVD(BaseEstimator, TransformerMixin):
             return x_scores, y_scores
         return x_scores
 
-    def fit_transform(self, X, y=None, **fit_params):
+    def fit_transform(self, X, y=None):
         """Learn and apply the dimension reduction on the train data.
 
         Parameters
         ----------
-        X : array-like of predictors, shape = [n_samples, p]
-            Training vectors, where n_samples in the number of samples and
-            p is the number of predictors.
+        X : array-like, shape = [n_samples, n_features]
+            Training vectors, where n_samples is the number of samples and
+            n_features is the number of predictors.
 
-        Y : array-like of response, shape = [n_samples, q], optional
-            Training vectors, where n_samples in the number of samples and
-            q is the number of response variables.
+        y : array-like, shape = [n_samples, n_targets]
+            Target vectors, where n_samples is the number of samples and
+            n_targets is the number of response variables.
 
         Returns
         -------
         x_scores if Y is not given, (x_scores, y_scores) otherwise.
         """
-        return self.fit(X, y, **fit_params).transform(X, y)
+        return self.fit(X, y).transform(X, y)
